//  
//       ,---.     ,--,    .---.     ,--,    ,---.    .-. .-. 
//       | .-'   .' .')   / .-. )  .' .'     | .-'    |  \| | 
//       | `-.   |  |(_)  | | |(_) |  |  __  | `-.    |   | | 
//       | .-'   \  \     | | | |  \  \ ( _) | .-'    | |\  | 
//       |  `--.  \  `-.  \ `-' /   \  `-) ) |  `--.  | | |)| 
//       /( __.'   \____\  )---'    )\____/  /( __.'  /(  (_) 
//      (__)              (_)      (__)     (__)     (__)     
//
//  This file is part of ECOGEN.
//
//  ECOGEN is the legal property of its developers, whose names 
//  are listed in the copyright file included with this source 
//  distribution.
//
//  ECOGEN is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published 
//  by the Free Software Foundation, either version 3 of the License, 
//  or (at your option) any later version.
//  
//  ECOGEN is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with ECOGEN (file LICENSE).  
//  If not, see <http://www.gnu.org/licenses/>.

//! \file      CellInterfaceO2.cpp
//! \author    F. Petitpas, K. Schmidmayer
//! \version   1.0
//! \date      December 20 2017

#include "CellInterfaceO2.h"
#include <iostream>

using namespace std;

Phase **slopesPhasesLocal1;
Phase **slopesPhasesLocal2;
Mixture *slopesMixtureLocal1;
Mixture *slopesMixtureLocal2;
double *slopesTransportLocal1;
double *slopesTransportLocal2;

//***********************************************************************

CellInterfaceO2::CellInterfaceO2() : CellInterface(), m_vecPhasesSlopes(0), m_mixtureSlopes(0), m_vecTransportsSlopes(0)
 //m_BG1M(0), m_BG2M(0), m_BG3M(0), m_BG1P(0), m_BG2P(0), m_BG3P(0), m_BD1M(0),
 //m_BD2M(0), m_BD3M(0), m_BD1P(0), m_BD2P(0), m_BD3P(0),
 //m_betaG1M(1.), m_betaG2M(0.), m_betaG3M(0.), m_betaG1P(1.), m_betaG2P(0.), m_betaG3P(0.),
 //m_betaD1M(1.), m_betaD2M(0.), m_betaD3M(0.), m_betaD1P(1.), m_betaD2P(0.), m_betaD3P(0.),
 //m_distanceHGM(0.), m_distanceHGP(0.), m_distanceHDM(0.), m_distanceHDP(0.)
 {}

//***********************************************************************

CellInterfaceO2::CellInterfaceO2(int lvl) : CellInterface(lvl), m_vecPhasesSlopes(0), m_mixtureSlopes(0), m_vecTransportsSlopes(0)
//m_BG1M(0), m_BG2M(0), m_BG3M(0), m_BG1P(0), m_BG2P(0), m_BG3P(0), m_BD1M(0),
//m_BD2M(0), m_BD3M(0), m_BD1P(0), m_BD2P(0), m_BD3P(0),
//m_betaG1M(1.), m_betaG2M(0.), m_betaG3M(0.), m_betaG1P(1.), m_betaG2P(0.), m_betaG3P(0.),
//m_betaD1M(1.), m_betaD2M(0.), m_betaD3M(0.), m_betaD1P(1.), m_betaD2P(0.), m_betaD3P(0.),
//m_distanceHGM(0.), m_distanceHGP(0.), m_distanceHDM(0.), m_distanceHDP(0.)
{}

//***********************************************************************

CellInterfaceO2::~CellInterfaceO2()
{
  for (int k = 0; k < m_numberPhases; k++) {
    delete m_vecPhasesSlopes[k];
  }
  delete[] m_vecPhasesSlopes;
  delete m_mixtureSlopes;
  delete[] m_vecTransportsSlopes;
}

//***********************************************************************

void CellInterfaceO2::allocateSlopes(const int &numberPhases, const int &numberTransports, int &allocateSlopeLocal)
{
  m_numberPhases = numberPhases;

  //Allocation des slopes des phases
	m_vecPhasesSlopes = new Phase*[numberPhases];
  //On attribut les phases a partir de la cell a gauche (car cell a droite inexistante pour les limites)
  //Necessaire car il faut connaitre le type de phase (ex: PhaseKapila, etc.))
  //Ensuite on met a zero toutes les slopes
  for(int k = 0; k < numberPhases; k++){
    m_cellLeft->getPhase(k)->allocateAndCopyPhase(&m_vecPhasesSlopes[k]);
		m_vecPhasesSlopes[k]->setToZero();
  }
  m_cellLeft->getMixture()->allocateAndCopyMixture(&m_mixtureSlopes);
  m_mixtureSlopes->setToZero();

	//Allocation des slopes sur transports
	m_vecTransportsSlopes = new Transport[numberTransports];
	for (int k = 0; k < numberTransports; k++) {
		m_vecTransportsSlopes[k].setValue(0.);
	}

  //Allocation des variables externes
  if (allocateSlopeLocal < 1) {
    slopesPhasesLocal1 = new Phase*[numberPhases];
    slopesPhasesLocal2 = new Phase*[numberPhases];
    for (int k = 0; k < numberPhases; k++) {
      m_cellLeft->getPhase(k)->allocateAndCopyPhase(&slopesPhasesLocal1[k]);
      m_cellLeft->getPhase(k)->allocateAndCopyPhase(&slopesPhasesLocal2[k]);
      slopesPhasesLocal1[k]->setToZero();
      slopesPhasesLocal2[k]->setToZero();
    }

    m_cellLeft->getMixture()->allocateAndCopyMixture(&slopesMixtureLocal1);
    m_cellLeft->getMixture()->allocateAndCopyMixture(&slopesMixtureLocal2);
    slopesMixtureLocal1->setToZero();
    slopesMixtureLocal2->setToZero();

		slopesTransportLocal1 = new double[numberTransports];
		slopesTransportLocal2 = new double[numberTransports];
		for (int k = 0; k < numberTransports; k++) {
			slopesTransportLocal1[k] = 0.;
			slopesTransportLocal2[k] = 0.;
		}

    allocateSlopeLocal = 1;
  }
}

//***********************************************************************

void CellInterfaceO2::computeSlopes(const int &numberPhases, const int &numberTransports, Prim type)
{
  if (m_boundariesChildren.size() == 0) {
    //Distance entre les deux mailles en contact
    double distance(m_cellLeft->distance(m_cellRight));
    //Attribution gauche/droite
    //Si la cell gauche ou droite est de niveau inferieur a "lvl", on ne prend pas "type" mais vecPhases (ca evite de prendre vecPhaseO2 alors qu'on ne l'a pas).
    Prim typeGauche = type;
    Prim typeDroite = type;
    if (m_cellLeft->getLvl() < m_lvl) { typeGauche = vecPhases; }
    if (m_cellRight->getLvl() < m_lvl) { typeDroite = vecPhases; }

    for (int k = 0; k < numberPhases; k++) {
			m_vecPhasesSlopes[k]->computeSlopesPhase(*m_cellLeft->getPhase(k, typeGauche), *m_cellRight->getPhase(k, typeDroite), distance);
    }
    m_mixtureSlopes->computeSlopesMixture(*m_cellLeft->getMixture(typeGauche), *m_cellRight->getMixture(typeDroite), distance);
    for (int k = 0; k < numberTransports; k++) {
      m_vecTransportsSlopes[k].computeSlopeTransport(m_cellLeft->getTransport(k, typeGauche).getValue(), m_cellRight->getTransport(k, typeDroite).getValue(), distance);
    }
  }
}

//***********************************************************************

void CellInterfaceO2::computeFlux(const int &numberPhases, const int &numberTransports, double &dtMax, Limiter &globalLimiter, Limiter &interfaceLimiter, Limiter &globalVolumeFractionLimiter, Limiter &interfaceVolumeFractionLimiter, Prim type)
{
  // Quand on fait le premier computeFlux (donc avec vecPhases) on n'incremente pas m_cons pour les mailles de niveau different (inferieur) de "lvl".
  // Sinon ca veut dire qu on l ajoute pour les 2 computeFlux sans le remettre a zero entre les deux, donc 2 fois plus de flux que ce que l on veut.
  this->solveRiemann(numberPhases, numberTransports, dtMax, globalLimiter, interfaceLimiter, globalVolumeFractionLimiter, interfaceVolumeFractionLimiter, type);

  switch (type) {
  case vecPhases:
    if (m_cellLeft->getLvl() == m_cellRight->getLvl()) {       //CoefAMR = 1 pour les deux
      this->addFlux(numberPhases, numberTransports, 1.);       //Ajout du flux sur maille droite
      this->subtractFlux(numberPhases, numberTransports, 1.);  //Retrait du flux sur maille gauche
    }
    else if (m_cellLeft->getLvl() > m_cellRight->getLvl()) {   //CoefAMR = 1 pour la gauche et on n'ajoute rien sur la maille droite
      this->subtractFlux(numberPhases, numberTransports, 1.);  //Retrait du flux sur maille gauche
    }
    else {                                                     //CoefAMR = 1 pour la droite et on ne retire rien sur la maille gauche
      this->addFlux(numberPhases, numberTransports, 1.);       //Ajout du flux sur maille droite
    }
    break;

  case vecPhasesO2:
    if (m_cellLeft->getLvl() == m_cellRight->getLvl()) {       //CoefAMR = 1 pour les deux
      this->addFlux(numberPhases, numberTransports, 1.);       //Ajout du flux sur maille droite
      this->subtractFlux(numberPhases, numberTransports, 1.);  //Retrait du flux sur maille gauche
    }
    else if (m_cellLeft->getLvl() > m_cellRight->getLvl()) {   //CoefAMR = 1 pour la gauche et 0.5 pour la droite
      this->addFlux(numberPhases, numberTransports, 0.5);      //Ajout du flux sur maille droite
      this->subtractFlux(numberPhases, numberTransports, 1.);  //Retrait du flux sur maille gauche
    }
    else {                                                     //CoefAMR = 1 pour la droite et 0.5 pour la gauche
      this->addFlux(numberPhases, numberTransports, 1.);       //Ajout du flux sur maille droite
      this->subtractFlux(numberPhases, numberTransports, 0.5); //Retrait du flux sur maille gauche
    }
    break;

  default: break;
  }
}

//***********************************************************************

void CellInterfaceO2::solveRiemann(const int &numberPhases, const int &numberTransports, double &dtMax, Limiter &globalLimiter, Limiter &interfaceLimiter, Limiter &globalVolumeFractionLimiter, Limiter &interfaceVolumeFractionLimiter, Prim type)
{
  //Si la cell gauche ou droite est de niveau inferieur a "lvl", on ne prend pas "type" mais vecPhases (ca evite de prendre vecPhaseO2 alors qu'on ne l'a pas).
  if (m_cellLeft->getLvl() == m_lvl) { cellLeft->copyVec(m_cellLeft->getPhases(type), m_cellLeft->getMixture(type), m_cellLeft->getTransports(type)); }
  else { cellLeft->copyVec(m_cellLeft->getPhases(vecPhases), m_cellLeft->getMixture(vecPhases), m_cellLeft->getTransports(vecPhases)); }
  
  if (m_cellRight->getLvl() == m_lvl) { cellRight->copyVec(m_cellRight->getPhases(type), m_cellRight->getMixture(type), m_cellRight->getTransports(type)); }
  else { cellRight->copyVec(m_cellRight->getPhases(vecPhases), m_cellRight->getMixture(vecPhases), m_cellRight->getTransports(vecPhases)); }

  //Calcul des distances bord de maille <-> cells pour l extrapolation
  double distanceGauche(this->distance(m_cellLeft));
  double distanceDroite(this->distance(m_cellRight));

  //Initialization of variables to detect the interface for limiters and THINC method
  int phase0(0), phase1(1);
  double alphaCellLeft(0.), alphaCellLeftLeft(0.), alphaCellRight(0.), alphaCellRightRight(0.);
  double beta(1.6), sign(0.), newAlpha(0.), A(0.), B(0.), C(0.), qmin(0.), qmax(0.), epsInterface(1.e-4);
  alphaCellLeft = cellLeft->getPhase(phase0)->getAlpha();
  alphaCellRight = cellRight->getPhase(phase0)->getAlpha();

  //Extrapolation gauche
  m_cellLeft->computeLocalSlopes(numberPhases, numberTransports, *this, globalLimiter, interfaceLimiter, globalVolumeFractionLimiter, interfaceVolumeFractionLimiter, alphaCellLeftLeft, alphaCellLeft, alphaCellRight, epsInterface);
  for (int k = 0; k < numberPhases; k++) {
    cellLeft->getPhase(k)->extrapolate(*slopesPhasesLocal1[k], distanceGauche);
    cellLeft->getPhase(k)->verifyAndCorrectPhase();
  }
  cellLeft->getMixture()->extrapolate(*slopesMixtureLocal1, distanceGauche);
	for (int k = 0; k < numberTransports; k++) {
		cellLeft->getTransport(k).extrapolate(slopesTransportLocal1[k], distanceGauche);
	}
  //THINC method (for alpha only)
  if (globalVolumeFractionLimiter.AmITHINC() || interfaceVolumeFractionLimiter.AmITHINC()) {
    if ((alphaCellLeft >= epsInterface) && (alphaCellLeft <= 1. - epsInterface) && ((alphaCellRight - alphaCellLeft)*(alphaCellLeft - alphaCellLeftLeft) >= 1.e-8)) {
      if (alphaCellRight - alphaCellLeftLeft > 0.) { sign = 1.; }
      else { sign = -1.; }
      qmin = min(alphaCellRight, alphaCellLeftLeft);
      qmax = max(alphaCellRight, alphaCellLeftLeft) - qmin;
      C = (alphaCellLeft - qmin + 1.e-20) / (qmax + 1.e-20);
      B = exp(sign*beta*(2.*C - 1.));
      A = (B / cosh(beta) - 1.) / tanh(beta);
      newAlpha = qmin + 0.5*qmax*(1. + sign * (tanh(beta) + A) / (1. + A * tanh(beta)));
      if (newAlpha < epsInterface) { newAlpha = epsInterface; }
      if (newAlpha > 1. - epsInterface) { newAlpha = 1. - epsInterface; }
      cellLeft->getPhase(phase0)->setAlpha(newAlpha);
      cellLeft->getPhase(phase1)->setAlpha(1. - newAlpha);
    }
  }

  //Extrapolation droite
  m_cellRight->computeLocalSlopes(numberPhases, numberTransports, *this, globalLimiter, interfaceLimiter, globalVolumeFractionLimiter, interfaceVolumeFractionLimiter, alphaCellRightRight, alphaCellRight, alphaCellLeft, epsInterface);
  for (int k = 0; k < numberPhases; k++) {
    slopesPhasesLocal1[k]->changeSign(); //On doit soustraire les slopes a droite
    cellRight->getPhase(k)->extrapolate(*slopesPhasesLocal1[k], distanceDroite);
    cellRight->getPhase(k)->verifyAndCorrectPhase();
  }
  slopesMixtureLocal1->changeSign();
  cellRight->getMixture()->extrapolate(*slopesMixtureLocal1, distanceDroite);
	for (int k = 0; k < numberTransports; k++) {
		slopesTransportLocal1[k] = -slopesTransportLocal1[k];
		cellRight->getTransport(k).extrapolate(slopesTransportLocal1[k], distanceDroite);
	}
  //THINC method (for alpha only)
  if (globalVolumeFractionLimiter.AmITHINC() || interfaceVolumeFractionLimiter.AmITHINC()) {
    if ((alphaCellRight >= epsInterface) && (alphaCellRight <= 1. - epsInterface) && ((alphaCellRightRight - alphaCellRight)*(alphaCellRight - alphaCellLeft) >= 1.e-8)) {
      if (alphaCellRightRight - alphaCellLeft > 0.) { sign = 1.; }
      else { sign = -1.; }
      qmin = min(alphaCellRightRight, alphaCellLeft);
      qmax = max(alphaCellRightRight, alphaCellLeft) - qmin;
      C = (alphaCellRight - qmin + 1.e-20) / (qmax + 1.e-20);
      B = exp(sign*beta*(2.*C - 1.));
      A = (B / cosh(beta) - 1.) / tanh(beta);
      newAlpha = qmin + 0.5*qmax*(1. + sign * A);
      if (newAlpha < epsInterface) { newAlpha = epsInterface; }
      if (newAlpha > 1. - epsInterface) { newAlpha = 1. - epsInterface; }
      cellRight->getPhase(phase0)->setAlpha(newAlpha);
      cellRight->getPhase(phase1)->setAlpha(1. - newAlpha);
    }
  }

  //Projection des velocities sur repere attache a la face
  cellLeft->localProjection(m_face->getNormal(), m_face->getTangent(), m_face->getBinormal(), numberPhases);
  cellRight->localProjection(m_face->getNormal(), m_face->getTangent(), m_face->getBinormal(), numberPhases);

  //Calcul des variables etendus (Phases, Mixture, AddPhys)
  cellLeft->fulfillState();
  cellRight->fulfillState();

  //Probleme de Riemann
  double dxLeft(m_cellLeft->getElement()->getLCFL());
  double dxRight(m_cellRight->getElement()->getLCFL());
  dxLeft = dxLeft*pow(2., (double)m_lvl);
  dxRight = dxRight*pow(2., (double)m_lvl);
  m_mod->solveRiemannIntern(*cellLeft, *cellRight, numberPhases, dxLeft, dxRight, dtMax);
  //Traitement des fonctions de transport (m_Sm connu : doit etre place apres l appel au Solveur de Riemann)
  if (numberTransports > 0) { m_mod->solveRiemannTransportIntern(*cellLeft, *cellRight, numberTransports); }

  //Projection du flux sur le repere absolu
  m_mod->reverseProjection(m_face->getNormal(), m_face->getTangent(), m_face->getBinormal());
}

//***********************************************************************

Phase* CellInterfaceO2::getSlopesPhase(const int &phaseNumber) const
{
  return m_vecPhasesSlopes[phaseNumber];
}

//***********************************************************************

Mixture* CellInterfaceO2::getSlopesMixture() const
{
  return m_mixtureSlopes;
}

//***********************************************************************

Transport* CellInterfaceO2::getSlopesTransport(const int &numberTransport) const
{
  return &m_vecTransportsSlopes[numberTransport];
}

//****************************************************************************
//******************************Methode AMR***********************************
//****************************************************************************

void CellInterfaceO2::creerBordChild()
{
  m_boundariesChildren.push_back(new CellInterfaceO2(m_lvl + 1));
}

//***********************************************************************

void CellInterfaceO2::creerBordChildInterne(const int &lvl, vector<CellInterface*> *childrenInternalBoundaries)
{
  (*childrenInternalBoundaries).push_back(new CellInterfaceO2(lvl + 1));
}

//***********************************************************************